/**
 * @file app.js
 * @autor Daniel Atienza Lopez
 * @date 02-05-2018
 */

const RINKEBY_ID = 4;

var ipfs = null;
var account = Object.assign({}, account_tmp);
var contract = null;

class App extends React.Component {
	constructor(props) {
		super(props);

		this.state = {
			error: null,
			error_msg: "",
			account: null,
			display: {
				user: null,
				admin: null,
				organization: null,
				student: null
			}
		};

		this.updateOrgForm = this.updateOrgForm.bind(this);
		this.updateStdForm = this.updateStdForm.bind(this);
		this.updateUserInfo = this.updateUserInfo.bind(this);
	}
	updateOrgForm() {
		if (account.viewer)
			return;

		this.organizationCmp.loadFormOrganization();
	}
	updateStdForm() {
		this.studentCmp.loadFormStudent();
	}
	updateUserInfo(type) {
		// User type Admin (1) or Organization (2)
		if (type == 1 || type == 2 || type == 3)
			this.userCmp.displayUserData();
	}
	exceptionError(err, ecode, eclass) {
		this.setState({
			error: ecode,
			error_msg: error_msg[ecode],
			error_class: eclass,
			process: {
				active: false
			}
		});
		if (err != null)
			throw err;
	}
	componentDidMount() {
		if (typeof config === 'undefined') {
			this.setState({
				error: '0x001',
				error_msg: error_msg['0x001']
			});
			return;
		}

		// Setup the IPFS config
		ipfs = IpfsApi({ host:config[config.mode].ipfs.host, port:config[config.mode].ipfs.port, protocol:config[config.mode].ipfs.protocol });

		if (typeof web3 !== 'undefined') {
			web3.version.getNetwork((err, result) => {
				if (err != null) {
					this.exceptionError(err,'0x008','error');
					return;
				}

				if (result == config[config.mode].ethereum.network_id) {
					window.web3 = new Web3(web3.currentProvider);
					if (web3.currentProvider.isMetaMask === true) {

						if (typeof this.props.address !== 'undefined'
								&& this.props.address != null) {
							account.address = this.props.address;
							account.viewer = true;
						}
						else {
							account.address = web3.eth.defaultAccount;
						}

						if (!web3.isAddress(account.address)) {
							this.setState({
								error: '0x104',
								error_msg: error_msg['0x104']
							})
							return;
						}

						web3.eth.getBalance(web3.eth.defaultAccount, (err, balance) => {
							account.balance = parseFloat(web3.fromWei(balance, "ether")).toFixed(4) + " ETH";

							var str_address = web3.eth.defaultAccount;
							if (typeof config[config.mode].ethereum.etherscan !== 'undefined')
								str_address = str_address.concat(' <a href="' + config[config.mode].ethereum.etherscan.replace('{{type}}','address').replace('{{id}}',account.address) + '" target="_blank"><i class="fas fa-external-link-alt"></i></a>');

							document.getElementById('account-address').innerHTML = str_address;
							document.getElementById('account-balance').innerHTML = account.balance;
						});

						contract = web3.eth.contract(config[config.mode].ethereum.contract.ABI).at(config[config.mode].ethereum.contract.address);

						contract._users(account.address, (err, result) => {
							account.type.code = web3.toDecimal(result);

							if (account.type.code == 0) {
								// User doesn't exists in the contract
								this.setState({
									display: {
										user: true
									}
								});
								return;
							}

							// User exists in the contract
							account.exists = true;
							account.type.name = T_USERS[account.type.code];

							if (account.type.code == T_ADMIN) {
								// Display user data (Admin)
								this.setState({ display: { user: true, admin: true }});
								return;
							}
							else if (account.type.code == T_UNIVERSITY) {
								// Display user data (Organization)
								this.setState({ display: { user: true, organization: true }});
								return;
							}
							else if (account.type.code == T_STUDENT) {
								// Display user data (Student)
								this.setState({ display: { user: true, student: true }});
								return;
							}
						});
					}
					else {
						this.setState({
							error: '0x103',
							error_msg: error_msg['0x103']
						});
					}
				}
				else {
					this.setState({
						error: '0x102',
						error_msg: error_msg['0x102'].replace('{{network_name}}', config[config.mode].ethereum.network_name)
					});
				}
			});
		}
		else {
			this.setState({
				error: '0x101',
				error_msg: error_msg['0x101']
			});
		}
	}
	componentWillUnmount() {}
	render() {
		return (
			<div>
				{(this.state.error == null) ? (
					<div class="container">
						{this.state.display.user &&
							<UserData
								ref={userCmp => { this.userCmp = userCmp; }}
								updateOrgForm={this.updateOrgForm}
								updateStdForm={this.updateStdForm}
							/>}
						{account.viewer == false && this.state.display.admin &&
							<AdminDashboard
								updateUserInfo={this.updateUserInfo}
							/>}
						{account.viewer == false && this.state.display.organization &&
							<OrganizationDashboard
								ref={organizationCmp => { this.organizationCmp = organizationCmp; }}
								updateUserInfo={this.updateUserInfo}
							/>}
						{this.state.display.student &&
							<StudentDashboard
								ref={studentCmp => { this.studentCmp = studentCmp; }}
								updateUserInfo={this.updateUserInfo}
							/>}
					</div>
				) : (
					<div class="container">
						<div class="row">
							<div
								class="info-box error"
								dangerouslySetInnerHTML={{__html: this.state.error_msg}}
							/>
						</div>
					</div>
				)}
			</div>
		);
	}
};

var address = getURLParameter("address");

ReactDOM.render((
    <App address={address} />
), document.getElementById('root'));
