/**
 * @file adminDashboard.jsx
 * @autor Daniel Atienza Lopez
 * @date 12-05-2018
 */

/**
* AdminDashboard Component
*/
class AdminDashboard extends React.Component {
	constructor(props) {
		super(props);

		this.state = {
			error: null,
			error_msg: "",
			error_class: "",
			process: {
				active: false,
				transaction: "",
				interval: null,
			}
		};

    this.exceptionError = this.exceptionError.bind(this);
    this.checkTransaction = this.checkTransaction.bind(this);
		this.addOrganization = this.addOrganization.bind(this);
	}
	componentDidMount() {}
	componentWillUnmount() {}
	checkTransaction() {
	  web3.eth.getTransactionReceipt(this.state.process.transaction, (err, result) => {
	    if (err) {
				clearInterval(this.state.process.interval);
				this.exceptionError(err, "0x003", "error");
				return;
	    }

	    if (result != null) {
				clearInterval(this.state.process.interval);

				this.props.updateUserInfo(1);

				if (typeof result.status !== 'undefined'
						&& web3.toDecimal(result.status) == 1) {
					this.exceptionError(null, "0x002", "ok");
				} else {
					this.exceptionError(null, "0x003", "error");
				}
	    }
	  });
	}
	exceptionError(err, ecode, eclass) {
		this.setState({
			error: ecode,
			error_msg: error_msg[ecode],
			error_class: eclass,
			process: {
				active: false
			}
		});
		if (err != null)
			throw err;
	}
	addOrganization(event) {
		event.preventDefault();

		this.setState({
			error: null
		});

		const std_address = this.input_address.value;

    if (this.state.process.active) {
			// Currently exists a process running
			return;
    }

		if (!web3.isAddress(std_address)) {
      this.exceptionError(null, '0x202', 'error');
			return;
    }

		if (!std_address.startsWith("0x")) {
			this.exceptionError(null, '0x007', 'error');
			return;
		}

		this.setState({
			process: {
				active: true
			}
		});

		contract._users(std_address, (err, result) => {
			if (err) {
				this.exceptionError(err, '0x000', null);
				return;
			}

			if (web3.toDecimal(result) != 0) {
				this.setState({
					error: "0x203",
					error_msg: error_msg['0x203'],
					error_class: "error",
					process: {
						active: false
					}
				});
				return;
			}

			contract.addUser(std_address, (err, result) => {
				if (err) {
					this.exceptionError(err, '0x000', 'error');
					return;
				}

				this.state.process.transaction = result;
				if (typeof config[config.mode].ethereum.etherscan !== 'undefined')
					result = result.concat(' <a href="' + config[config.mode].ethereum.etherscan.replace('{{type}}','tx').replace('{{id}}',this.state.process.transaction) + '" target="_blank"><i class="fas fa-external-link-alt"></i></a>');
				var aux_error_msg = error_msg['0x201'].replace('{{tx}}',result);

				this.setState({
					error: '0x201',
					error_msg: aux_error_msg,
					error_class: "waiting"
				});

				this.state.process.interval = setInterval(this.checkTransaction, 5000);
			});
		});
	}
	render() {
		return (
			<div id="admin-dashboard">
				<div class="row">
					<h2>Afegir una nova organització</h2>
					{this.state.error != null &&
					<div
						class={'info-box ' + this.state.error_class}
						dangerouslySetInnerHTML={{__html: this.state.error_msg}}
					/>}
					<form onSubmit={this.addOrganization}>
						<label for="inputPublickKey">Clau publica</label>
						<input
							id="inputPublickKey"
							class="u-full-width"
							type="text" placeholder="0x..."
							disabled={this.state.process.active}
							ref={input => { this.input_address = input; }}
						/>
						<button
							class="button-primary"
							disabled={this.state.process.active}>
							<i class="fas fa-plus"></i> Afegir
						</button>
					</form>
				</div>
			</div>
		);
	}
};
