/**
 * @file organizationDashboard.jsx
 * @autor Daniel Atienza Lopez
 * @date 12-05-2018
 */

/**
* OrganizationDashboard Component
*/
class OrganizationDashboard extends React.Component {
	constructor(props) {
		super(props);

		this.state = {
			error_display: null,
			error: null,
			error_msg: "",
			error_class: "",
			process: {
				active: false,
				transaction: "",
				interval: null,
			}
		};

    this.updateOrganization = this.updateOrganization.bind(this);
    this.addStudent = this.addStudent.bind(this);
    this.addCertificate = this.addCertificate.bind(this);
    this.exceptionError = this.exceptionError.bind(this);
    this.checkTransaction = this.checkTransaction.bind(this);
    this.loadFormOrganization = this.loadFormOrganization.bind(this);
		this.clearForm = this.clearForm.bind(this);
	}
	exceptionError(display, ecode, err) {
		this.setState({
			error_display: display,
			error: ecode,
			error_msg: error_msg[ecode],
			error_class: "error",
			process: {
				active: false
			}
		});
		if (err != null)
			throw err;
	}
	checkTransaction() {
		web3.eth.getTransactionReceipt(this.state.process.transaction, (err, result) => {
			if (err) {
				clearInterval(this.state.process.interval);
				this.exceptionError(this.state.error_display, "0x003", err);
				return;
			}

			if (result != null) {
				clearInterval(this.state.process.interval);

				if (typeof result.status !== 'undefined'
						&& web3.toDecimal(result.status) == 1) {

					if (this.state.error_display == 1)
						this.props.updateUserInfo(2); // Action: Update Org data >> update user info
					else if (this.state.error_display == 2)
						this.clearForm("form_add_student"); // Action: Add Student >> Clean forms
					else if (this.state.error_display == 3) {
						this.clearForm("form_add_cert"); // Action: Add Certificate >> Clean forms
						this.props.updateUserInfo(2);
					}

					this.setState({
						error: "0x002",
						error_msg: error_msg["0x002"].replace("{{tx}}",this.state.process.transaction),
						error_class: "ok",
						process: {
							active: false
						}
					});
				} else {
					this.exceptionError(this.state.error_display, "0x003", null);
				}
			}
		});
	}
	loadFormOrganization() {
		this.input_orgname.value = account.user_object.name;
		this.input_orgphone.value = account.user_object.phone;
		this.input_orgemail.value = account.user_object.email;
		this.input_orgwebsite.value = account.user_object.website;
	}
	updateOrganization(event) {
		event.preventDefault();

		const n_display = 1;
		const ipfs_data = {
			address: account.address,
			name: this.input_orgname.value,
			phone: this.input_orgphone.value,
			email: this.input_orgemail.value,
			website: this.input_orgwebsite.value,
			added_timestamp: moment().unix()
		};

		this.setState({
			error: null
		});

		if (this.state.process.active) {
			// Currently exists a process running
			return;
		}

		if (ipfs_data.name.length == 0
				|| ipfs_data.phone.length == 0
				|| ipfs_data.email.length == 0
				|| ipfs_data.website.length == 0) {
			this.exceptionError(n_display, '0x005', null);
			return;
		}

		// Start the process - Disable forms
		this.setState({
			error_display: n_display,
			error: '0x006',
			error_msg: error_msg['0x006'],
			error_class: 'waiting',
			process: {
				active: true
			}
		});

    try {
  		ipfs.files.add(ipfs.Buffer(JSON.stringify(ipfs_data)), function (err, result) {
  			if (err || result == null) {
  				this.exceptionError(n_display, '0x004', err);
  				return;
  			}

  			const ipfs_hash = result[0].hash;
  			contract.setOrganization(ipfs_hash, (err, result) => {
  				if (err) {
  					this.exceptionError(n_display, '0x000', err);
  					return;
  				}

  				this.state.process.transaction = result;
  				if (typeof config[config.mode].ethereum.etherscan !== 'undefined')
  					result = result.concat(' <a href="' + config[config.mode].ethereum.etherscan.replace('{{type}}','tx').replace('{{id}}',this.state.process.transaction) + '" target="_blank"><i class="fas fa-external-link-alt"></i></a>');
  				var aux_error_msg = error_msg['0x201'].replace('{{tx}}',result);

  				this.setState({
  					error_display: n_display,
  					error: '0x201',
  					error_msg: aux_error_msg,
  					error_class: 'waiting'
  				});

  				this.state.process.interval = setInterval(this.checkTransaction, 5000);
  			});
  		}.bind(this));
    }
    catch (err) {
      this.exceptionError(n_display, '0x004', err);
      return;
    }
	}
	addStudent(event) {
		event.preventDefault();

		const n_display = 2;
		const student_address = this.input_staddress.value;

		this.setState({
			error: null
		});

		if (this.state.process.active) {
			// Currently exists a process running
			return;
		}

		if (!web3.isAddress(student_address)) {
			this.exceptionError(n_display, '0x301', null);
			return;
		}

		if (!student_address.startsWith("0x")) {
			this.exceptionError(n_display, '0x007', null);
			return;
		}

		this.setState({
			process: {
				active: true
			}
		});

		contract._users(student_address, (err, result) => {
			if (err) {
				this.exceptionError(n_display, '0x000', err);
				return;
			}

			if (web3.toDecimal(result) != 0) {
				this.setState({
					error_display: n_display,
					error: "0x302",
					error_msg: error_msg['0x302'],
					error_class: "error",
					process: {
						active: false
					}
				});
				return;
			}

			contract.addUser(student_address, (err, result) => {
				if (err) {
					this.exceptionError(n_display, '0x000', err);
					return;
				}

				this.state.process.transaction = result;
				if (typeof config[config.mode].ethereum.etherscan !== 'undefined')
					result = result.concat(' <a href="' + config[config.mode].ethereum.etherscan.replace('{{type}}','tx').replace('{{id}}',this.state.process.transaction) + '" target="_blank"><i class="fas fa-external-link-alt"></i></a>');
				var aux_error_msg = error_msg['0x201'].replace('{{tx}}',result);

				this.setState({
					error_display: n_display,
					error: '0x201',
					error_msg: aux_error_msg,
					error_class: "waiting"
				});

				this.state.process.interval = setInterval(this.checkTransaction, 5000);
			});
		});
	}
	addCertificate(event) {
		event.preventDefault();

		const n_display = 3;

		this.setState({ error: null });

		if (this.state.process.active) {
			// Currently exists a process running
			return;
		}

		if (this.input_certaddress.value.length == 0
				|| this.input_certprogram.value.length == 0
				|| this.input_certgrade.value == 0
				|| typeof this.input_certfile.files[0] === 'undefined') {
			this.exceptionError(n_display, '0x005', null);
			return;
		}

		if (!web3.isAddress(this.input_certaddress.value)) {
			this.exceptionError(n_display, '0x301', null);
			return;
		}

		if (!this.input_certaddress.value.startsWith("0x")) {
			this.exceptionError(n_display, '0x007', null);
			return;
		}

		this.setState({
			process: {
				active: true
			}
		});

		contract._users(this.input_certaddress.value, (err, result) => {
			if (err) {
				this.exceptionError(n_display, '0x000', err);
				return;
			}

			if (web3.toDecimal(result) == 0
					|| web3.toDecimal(result) != T_STUDENT) {
				this.exceptionError(n_display, '0x303', null);
				return;
			}

			this.setState({
				error_display: n_display,
				error: '0x006',
				error_msg: error_msg['0x006'],
				error_class: "waiting"
			});

			try {
				var cert_file = new Blob([this.input_certfile.files[0]]);
				var file_reader = new FileReader();
				file_reader.readAsArrayBuffer(cert_file);
				file_reader.onloadend = () => {
					ipfs.files.add(ipfs.Buffer(file_reader.result), function (err, result) {
						if (err || result == null) {
							this.exceptionError(n_display, '0x004', err);
							return;
						}

						const ipfs_data = {
							organization: {
                address: account.address,
                name: ((typeof account.user_object !== 'undefined' && typeof account.user_object.name !== 'undefined')?account.user_object.name:''),
              },
							student: this.input_certaddress.value,
							program: this.input_certprogram.value,
							grade: this.input_certgrade.value,
							file: result[0].hash,
							added_timestamp: moment().unix()
						};

            try {
  						ipfs.files.add(ipfs.Buffer(JSON.stringify(ipfs_data)), (err, result) => {
  							if (err || result == null) {
  								this.exceptionError(n_display, '0x004', err);
  								return;
  							}

  							this.setState({
  								error_display: n_display,
  								error: '0x006',
  								error_msg: error_msg['0x006'],
  								error_class: 'waiting'
  							});

  							const ipfs_hash = result[0].hash;
  							contract.addCertificate(this.input_certaddress.value, ipfs_hash, (err, result) => {
  									if (err) {
  										this.exceptionError(n_display, '0x000', err);
  										return;
  									}

  									this.state.process.transaction = result;
  									if (typeof config[config.mode].ethereum.etherscan !== 'undefined')
  										result = result.concat(' <a href="' + config[config.mode].ethereum.etherscan.replace('{{type}}','tx').replace('{{id}}',this.state.process.transaction) + '" target="_blank"><i class="fas fa-external-link-alt"></i></a>');
  									var aux_error_msg = error_msg['0x201'].replace('{{tx}}',result);

  									this.setState({
  										error_display: n_display,
  										error: '0x201',
  										error_msg: aux_error_msg,
  										error_class: 'waiting'
  									});

  									this.state.process.interval = setInterval(this.checkTransaction, 5000);
  							});
  						});
            } catch (err) {
              this.exceptionError(n_display, '0x004', err);
              return;
            }

					}.bind(this));
				};
			}
			catch (err) {
				this.exceptionError(n_display, '0x000', err);
				return;
			}
		});
	}
	clearForm(form_id) {
		if (typeof this.refs[form_id] !== 'undefined')
			this.refs[form_id].reset();
	}
	componentDidMount() {}
	componentWillUnmount() {}
	render() {
		return (
			<div id="organization-dashboard">
				<div class="row">
					<h2>Actualitzar les dades de l'organització</h2>
					{this.state.error_display == 1 && this.state.error != null &&
					<div
						class={'info-box ' + this.state.error_class}
						dangerouslySetInnerHTML={{__html: this.state.error_msg}}
					/>}
					<form onSubmit={this.updateOrganization} ref="form_update_org">
						<label for="inputNameOrg">Nom de l'organització</label>
						<input
							id="inputNameOrg"
							class="u-full-width"
							type="text"
							disabled={this.state.process.active}
							ref={input => { this.input_orgname = input; }}
						/>
						<label for="inputPhone">Telefon</label>
						<input
							id="inputPhone"
							class="u-full-width"
							type="text"
							disabled={this.state.process.active}
							ref={input => { this.input_orgphone = input; }}
						/>
						<label for="inputEmail">Email</label>
						<input
							id="inputEmail"
							class="u-full-width"
							type="email"
							disabled={this.state.process.active}
							ref={input => { this.input_orgemail = input; }}
						/>
						<label for="inputWebsite">Pàgina web</label>
						<input
							id="inputWebsite"
							class="u-full-width"
							type="url"
							placeholder="https://..."
							disabled={this.state.process.active}
							ref={input => { this.input_orgwebsite = input; }}
						/>
						<button
							class="button-primary"
							disabled={this.state.process.active}>
							<i class="fas fa-plus"></i> Desar
						</button>
					</form>
				</div>
				<div class="row">
					<h2>Afegir un nou estudiant</h2>
					{this.state.error_display == 2 && this.state.error != null &&
					<div
						class={'info-box ' + this.state.error_class}
						dangerouslySetInnerHTML={{__html: this.state.error_msg}}
					/>}
					<form onSubmit={this.addStudent} ref="form_add_student">
						<label for="inputPublickKey1">Clau publica de l'alumne</label>
						<input
							id="inputPublickKey1"
							class="u-full-width"
							type="text"
							placeholder="0x..."
							disabled={this.state.process.active}
							ref={input => { this.input_staddress = input; }}
						/>
						<button
							class="button-primary"
							disabled={this.state.process.active}>
							<i class="fas fa-plus"></i> Afegir
						</button>
					</form>
				</div>
					<div class="row">
						<h2>Afegir un nou certificat</h2>
						{this.state.error_display == 3 && this.state.error != null &&
						<div
							class={'info-box ' + this.state.error_class}
							dangerouslySetInnerHTML={{__html: this.state.error_msg}}
						/>}
						<form onSubmit={this.addCertificate} ref="form_add_cert">
							<label for="inputPublickKey2">Clau publica de l'alumne</label>
							<input
								id="inputPublickKey2"
								class="u-full-width"
								type="text"
								placeholder="0x..."
								disabled={this.state.process.active}
								ref={input => { this.input_certaddress = input; }}
							/>
							<label for="inputNameProgram">Nom dels estudis</label>
							<input
								id="inputNameProgram"
								class="u-full-width"
								type="text"
								disabled={this.state.process.active}
								ref={input => { this.input_certprogram = input; }}
							/>
							<label for="inputQualification">Nota</label>
							<input
								id="inputQualification"
								class="u-full-width"
								type="text"
								disabled={this.state.process.active}
								ref={input => { this.input_certgrade = input; }}
							/>
							<label for="inputFileCert">Certificat</label>
							<input
								id="inputFileCert"
								class="u-full-width"
								type="file"
								disabled={this.state.process.active}
								ref={input => { this.input_certfile = input; }}
							/>
							<button
								class="button-primary"
								disabled={this.state.process.active}>
								<i class="fas fa-plus"></i> Desar
							</button>
						</form>
					</div>
			</div>
		);
	}
};
