/**
 * @file studentDashboard.jsx
 * @autor Daniel Atienza Lopez
 * @date 12-05-2018
 */

/**
* StudentDashboard Component
*/
class StudentDashboard extends React.Component {
	constructor(props) {
		super(props);

		this.state = {
			error: null,
			error_msg: "",
			error_class: "",
			display_cert: false,
			process: {
				active: false,
				transaction: "",
				interval: null,
			}
		};

		this.exceptionError = this.exceptionError.bind(this);
		this.checkTransaction = this.checkTransaction.bind(this);
		this.loadFormStudent = this.loadFormStudent.bind(this);
		this.updateStudent = this.updateStudent.bind(this);
	}
	exceptionError(err, ecode) {
		this.setState({
			error: ecode,
			error_msg: error_msg[ecode],
			error_class: "error",
			process: {
				active: false
			}
		});
		if (err != null)
			throw err;
	}
	checkTransaction() {
		web3.eth.getTransactionReceipt(this.state.process.transaction, (err, result) => {
			if (err) {
				clearInterval(this.state.process.interval);
				this.exceptionError(err, "0x003");
				return;
			}

			if (result != null) {
				clearInterval(this.state.process.interval);

				if (typeof result.status !== 'undefined'
						&& web3.toDecimal(result.status) == 1) {
					this.props.updateUserInfo(3);

					this.setState({
						error: "0x002",
						error_msg: error_msg["0x002"].replace("{{tx}}",this.state.process.transaction),
						error_class: "ok",
						process: {
							active: false
						}
					});
				} else {
					this.exceptionError(null, "0x003");
				}
			}
		});
	}
	loadFormStudent() {
		if (!account.viewer) {
			this.input_name.value = account.user_object.name;
			this.input_surname.value = account.user_object.surname;
		}

		for(var i = 0; i < account.user_object.qt_certificates; i++) {
			contract.getCertificate(account.address, i, (err, result) => {
				if (err) {
					this.exceptionError(err, '0x000');
					return;
				}

				var certificate = Object.assign({}, certificate_tmp);

				certificate.id = web3.toDecimal(result[0]);
				certificate.ipfs_hash = result[2];
				//certificate.program = "";
				//certificate.grade = "";
				//certificate.file = "";
				certificate.added_by_address = result[1];
				//certificate.added_by_name = "";
				certificate.added_timestamp = moment.unix(web3.toDecimal(result[3])).format("DD/MM/YYYY");

				ipfs.files.cat(result[2], (err, result) => {
					if (err) {
						this.exceptionError(err, '0x000');
						return;
					}

					var json_cert = {};
					try {
						json_cert = JSON.parse(result.toString('utf8'));

						if (typeof json_cert.organization !== 'undefined') {
							if (typeof json_cert.organization.name !== 'undefined')
								certificate.added_by_name = json_cert.organization.name;
						}

						if (typeof json_cert.program !== 'undefined')
							certificate.program = json_cert.program;

						if (typeof json_cert.grade !== 'undefined')
							certificate.grade = json_cert.grade;

						if (typeof json_cert.file !== 'undefined')
							certificate.file = json_cert.file;
					}
					catch (err) {}

					account.user_object.certificates.push(certificate);
					this.setState({ display_cert: true });
				});
			});
		}
	}
	updateStudent(event) {
		event.preventDefault();

		const ipfs_data = {
			address: account.address,
			name: this.input_name.value,
			surname: this.input_surname.value,
			added_timestamp: moment().unix()
		};

		this.setState({
			error: null
		});

		if (this.state.process.active) {
			// Currently exists a process running
			return;
		}

		if (ipfs_data.name.length == 0
				|| ipfs_data.surname.length == 0) {
			this.exceptionError(null, '0x005');
			return;
		}

		// Start the process - Disable forms
		this.setState({
			error: '0x006',
			error_msg: error_msg['0x006'],
			error_class: 'waiting',
			process: {
				active: true
			}
		});

		try {
  		ipfs.files.add(ipfs.Buffer(JSON.stringify(ipfs_data)), function (err, result) {
  			if (err || result == null) {
  				this.exceptionError(err, '0x004');
  				return;
  			}

  			const ipfs_hash = result[0].hash;
  			contract.setStudent(ipfs_hash, (err, result) => {
  				if (err) {
  					this.exceptionError(err, '0x000');
  					return;
  				}

  				this.state.process.transaction = result;
  				if (typeof config[config.mode].ethereum.etherscan !== 'undefined')
  					result = result.concat(' <a href="' + config[config.mode].ethereum.etherscan.replace('{{type}}','tx').replace('{{id}}',this.state.process.transaction) + '" target="_blank"><i class="fas fa-external-link-alt"></i></a>');
  				var aux_error_msg = error_msg['0x201'].replace('{{tx}}',result);

  				this.setState({
  					error: '0x201',
  					error_msg: aux_error_msg,
  					error_class: 'waiting'
  				});

  				this.state.process.interval = setInterval(this.checkTransaction, 5000);
				});
			}.bind(this));
		}
		catch (err) {
      this.exceptionError(err, '0x004');
      return;
		}
	}
	componentDidMount() {}
	componentWillUnmount() {}
	render() {
		return (
			<div id="student-dashboard">
				{account.viewer == false &&
				<div class="row">
					<h2>Actualitzar les dades de l'alumne</h2>
					{this.state.error != null &&
					<div
						class={'info-box ' + this.state.error_class}
						dangerouslySetInnerHTML={{__html: this.state.error_msg}}
					/>}
					<form onSubmit={this.updateStudent} ref="form_update_std">
						<label for="inputName">Nom de l'alumne</label>
						<input
							id="inputName"
							class="u-full-width"
							type="text"
							disabled={this.state.process.active}
							ref={input => { this.input_name = input; }}
						/>
						<label for="inputSurnames">Cognom/s de l'alumne</label>
						<input
							id="inputSurnames"
							class="u-full-width"
							type="text"
							disabled={this.state.process.active}
							ref={input => { this.input_surname = input; }}
						/>
						<button
							class="button-primary"
							disabled={this.state.process.active}>
							<i class="fas fa-plus"></i> Desar
						</button>
					</form>
				</div>}
				{this.state.display_cert &&
				<div class="row">
					<h2>Certificats</h2>
					<table id="tbl-certificates" class="u-full-width">
					  <thead>
					    <tr>
								<th>Organització</th>
					      <th>Estudis</th>
					      <th>Nota</th>
					      <th>Afegit</th>
					      <th>Certificat</th>
					    </tr>
					  </thead>
					  <tbody>
							{account.user_object.certificates.map((cert_obj) => {
								var html_certificate = (
						    	<tr>
										<td><a href={window.location.origin + window.location.pathname + '?address=' + cert_obj.added_by_address} target="_blank">{cert_obj.added_by_name}</a></td>
										<td>{cert_obj.program}</td>
										<td>{cert_obj.grade}</td>
										<td>{cert_obj.added_timestamp}</td>
										<td>
											<ul>
												<li><a href={'https://ipfs.io/ipfs/' + cert_obj.file} target="_blank">Enllaç 1</a></li>
												<li><a href={'https://ipfs.infura.io/ipfs/' + cert_obj.file} target="_blank">Enllaç 2</a></li>
												<li><a href={'https://siderus.io/ipfs/' + cert_obj.file} target="_blank">Enllaç 3</a></li>
												{ config.mode == 'test' &&
												<li><a href={config[config.mode].ipfs.protocol + '://' + config[config.mode].ipfs.host + ':' + config[config.mode].ipfs.port_web + '/ipfs/' + cert_obj.file} target="_blank">Enllaç 4 (Local)</a></li>}
											</ul>
										</td>
									</tr>
								);
								return html_certificate;
							})}
					  </tbody>
					</table>
				</div>}
			</div>
		);
	}
};
